@@ -34,7 +34,7 @@ where
         }
 
         if negate_cost(&stmt.test, true, false) < 0 {
-            tracing::debug!("if_return: Negating `cond` of an if statement which has cons and alt");
+            report_change!("if_return: Negating `cond` of an if statement which has cons and alt");
             let ctx = Ctx {
                 in_bool_ctx: true,
                 ..self.ctx
@@ -47,7 +47,7 @@ where
         match &*alt {
             Stmt::Return(..) | Stmt::Continue(ContinueStmt { label: None, .. }) => {
                 self.changed = true;
-                tracing::debug!(
+                report_change!(
                     "if_return: Negating an if statement because the alt is return / continue"
                 );
                 self.negate(&mut stmt.test, false);
@@ -68,7 +68,7 @@ where
 
         if !cond.cons.may_have_side_effects() {
             self.changed = true;
-            tracing::debug!("conditionals: `cond ? useless : alt` => `cond || alt`");
+            report_change!("conditionals: `cond ? useless : alt` => `cond || alt`");
             *e = Expr::Bin(BinExpr {
                 span: cond.span,
                 op: op!("||"),
@@ -80,7 +80,7 @@ where
 
         if !cond.alt.may_have_side_effects() {
             self.changed = true;
-            tracing::debug!("conditionals: `cond ? cons : useless` => `cond && cons`");
+            report_change!("conditionals: `cond ? cons : useless` => `cond && cons`");
             *e = Expr::Bin(BinExpr {
                 span: cond.span,
                 op: op!("&&"),
@@ -127,7 +127,7 @@ where
         }
 
         self.changed = true;
-        tracing::debug!("conditionals: Merging if statements with same `cons`");
+        report_change!("conditionals: Merging if statements with same `cons`");
 
         let mut cur: Option<IfStmt> = None;
         let mut new = Vec::with_capacity(stmts.len());
@@ -150,6 +150,8 @@ where
                                         }));
                                     } else {
                                         new.extend(cur.take().map(Stmt::If).map(T::from_stmt));
+
+                                        cur = Some(stmt);
                                     }
                                 }
                                 None => {
@@ -212,7 +214,7 @@ where
                     expr: stmt.test.take(),
                 });
                 self.changed = true;
-                tracing::debug!("conditionals: `if (foo);` => `foo` ");
+                report_change!("conditionals: `if (foo);` => `foo` ");
                 return;
             }
         }
@@ -238,7 +240,7 @@ where
                 Expr::Unary(UnaryExpr {
                     op: op!("!"), arg, ..
                 }) => {
-                    tracing::debug!("Optimizing `if (!foo); else bar();` as `foo && bar();`");
+                    report_change!("Optimizing `if (!foo); else bar();` as `foo && bar();`");
 
                     let mut expr = Box::new(Expr::Bin(BinExpr {
                         span: DUMMY_SP,
@@ -253,7 +255,7 @@ where
                     });
                 }
                 _ => {
-                    tracing::debug!("Optimizing `if (foo); else bar();` as `foo || bar();`");
+                    report_change!("Optimizing `if (foo); else bar();` as `foo || bar();`");
 
                     let mut expr = Box::new(Expr::Bin(BinExpr {
                         span: DUMMY_SP,
@@ -288,7 +290,7 @@ where
 
         if self.options.conditionals || self.options.bools {
             // if (a) b(); else c(); => a ? b() : c()
-            tracing::debug!(
+            report_change!(
                 "Compressing if statement as conditional expression (even though cons and alt is \
                  not compressable)"
             );
@@ -323,7 +325,7 @@ where
 
         // x ? x : y => x || y
         if cond.test.is_ident() && cond.test.eq_ignore_span(&cond.cons) {
-            tracing::debug!("Compressing `x ? x : y` as `x || y`");
+            report_change!("Compressing `x ? x : y` as `x || y`");
             self.changed = true;
             *e = Expr::Bin(BinExpr {
                 span: cond.span,
@@ -342,7 +344,7 @@ where
         is_for_if_stmt: bool,
     ) -> Option<Expr> {
         if cons.eq_ignore_span(alt) && !matches!(&*cons, Expr::Yield(..) | Expr::Fn(..)) {
-            tracing::debug!("conditionals: cons is same as alt");
+            report_change!("conditionals: cons is same as alt");
             return Some(Expr::Seq(SeqExpr {
                 span: DUMMY_SP,
                 exprs: vec![test.take(), Box::new(cons.take())],
@@ -382,7 +384,7 @@ where
                         .count();
 
                     if diff_count == 1 {
-                        tracing::debug!(
+                        report_change!(
                             "conditionals: Merging cons and alt as only one argument differs"
                         );
                         self.changed = true;
@@ -444,7 +446,7 @@ where
                     }
                     .as_arg()];
 
-                    tracing::debug!(
+                    report_change!(
                         "Compressing if into cond as there's no side effect and the number of \
                          arguments is 1"
                     );
@@ -457,7 +459,7 @@ where
                 }
 
                 if !side_effect_free && is_for_if_stmt {
-                    tracing::debug!("Compressing if into cond while preserving side effects");
+                    report_change!("Compressing if into cond while preserving side effects");
                     return Some(Expr::Cond(CondExpr {
                         span: DUMMY_SP,
                         test: test.take(),
@@ -510,7 +512,7 @@ where
                         });
                     }
 
-                    tracing::debug!(
+                    report_change!(
                         "Compressing if statement into a conditional expression of `new` as \
                          there's no side effect and the number of arguments is 1"
                     );
@@ -531,7 +533,7 @@ where
                     && cons.left.eq_ignore_span(&alt.left)
                     && is_simple_lhs(&cons.left) =>
             {
-                tracing::debug!("Merging assignments in cons and alt of if statement");
+                report_change!("Merging assignments in cons and alt of if statement");
                 Some(Expr::Assign(AssignExpr {
                     span: DUMMY_SP,
                     op: cons.op,
@@ -547,7 +549,7 @@ where
 
             // a ? b ? c() : d() : d() => a && b ? c() : d()
             (Expr::Cond(cons), alt) if (*cons.alt).eq_ignore_span(&*alt) => {
-                tracing::debug!("conditionals: a ? b ? c() : d() : d() => a && b ? c() : d()");
+                report_change!("conditionals: a ? b ? c() : d() : d() => a && b ? c() : d()");
                 Some(Expr::Cond(CondExpr {
                     span: DUMMY_SP,
                     test: Box::new(Expr::Bin(BinExpr {
@@ -566,7 +568,7 @@ where
             // (z || condition(), "fuji");
             (cons, Expr::Seq(alt)) if (**alt.exprs.last().unwrap()).eq_ignore_span(&*cons) => {
                 self.changed = true;
-                tracing::debug!("conditionals: Reducing seq expr in alt");
+                report_change!("conditionals: Reducing seq expr in alt");
                 //
                 alt.exprs.pop();
                 let first = Box::new(Expr::Bin(BinExpr {
@@ -589,7 +591,7 @@ where
             // (z && condition(), "fuji");
             (Expr::Seq(cons), alt) if (**cons.exprs.last().unwrap()).eq_ignore_span(&*alt) => {
                 self.changed = true;
-                tracing::debug!("conditionals: Reducing seq expr in cons");
+                report_change!("conditionals: Reducing seq expr in cons");
                 //
                 cons.exprs.pop();
                 let first = Box::new(Expr::Bin(BinExpr {
@@ -645,7 +647,7 @@ where
         };
 
         self.changed = true;
-        tracing::debug!("if_return: Injecting else because it's shorter");
+        report_change!("if_return: Injecting else because it's shorter");
 
         let mut new = vec![];
         new.reserve(pos_of_if + 1);
@@ -736,7 +738,7 @@ where
         }
 
         self.changed = true;
-        tracing::debug!("conditionals: Dropped useless `else` token");
+        report_change!("conditionals: Dropped useless `else` token");
         *stmts = new_stmts;
     }
 }
