@@ -5,8 +5,8 @@ use swc_atoms::js_word;
 use swc_common::{util::take::Take, Spanned, DUMMY_SP};
 use swc_ecma_ast::*;
 use swc_ecma_utils::{
-    contains_arguments, contains_this_expr, ident::IdentLike, undefined, ExprExt, Id, StmtLike,
-    UsageFinder,
+    contains_arguments, contains_this_expr, ident::IdentLike, prepend_stmts, undefined, ExprExt,
+    Id, StmtLike, UsageFinder,
 };
 use swc_ecma_visit::{noop_visit_type, Visit, VisitWith};
 use tracing::{span, Level};
@@ -139,7 +139,7 @@ where
             }
         }
 
-        tracing::debug!("sequences: Compressing statements as a sequences");
+        report_change!("sequences: Compressing statements as a sequences");
 
         self.changed = true;
         let mut exprs = vec![];
@@ -415,7 +415,7 @@ where
             }
         }
         self.changed = true;
-        tracing::debug!(
+        report_change!(
             "sequences: Splitted a sequence expression to multiple expression statements"
         );
         *stmts = new_stmts;
@@ -449,7 +449,7 @@ where
             if !can_work {
                 return;
             }
-            tracing::debug!("sequences: Lifting");
+            report_change!("sequences: Lifting");
             self.changed = true;
         }
 
@@ -483,74 +483,27 @@ where
         e.exprs = new_exprs;
     }
 
-    /// Hoist variables in subscope.
-    ///
-    /// I don't know why it depends on `sequences`.
-    pub(super) fn extract_vars_in_subscopes(&mut self, s: &mut Stmt) {
-        if !self.options.sequences() {
-            return;
+    #[allow(unused)]
+    pub(super) fn optimize_with_extras<T, F>(&mut self, stmts: &mut Vec<T>, mut op: F)
+    where
+        F: FnMut(&mut Vec<T>),
+        T: ModuleItemExt,
+    {
+        let old_prepend = self.prepend_stmts.take();
+        let old_append = self.append_stmts.take();
+
+        op(stmts);
+
+        if !self.prepend_stmts.is_empty() {
+            prepend_stmts(stmts, self.prepend_stmts.drain(..).map(T::from_stmt));
         }
 
-        match s {
-            Stmt::If(stmt) if self.options.conditionals => {
-                self.extract_vars(&mut stmt.cons);
-                if let Some(alt) = &mut stmt.alt {
-                    self.extract_vars(alt);
-                }
-            }
-
-            _ => {}
+        if self.append_stmts.is_empty() {
+            stmts.extend(self.append_stmts.drain(..).map(T::from_stmt));
         }
-    }
 
-    /// Move `var` in subscope to current scope.
-    ///
-    /// This method actually `hoist`s [VarDecl]s declared with `var`.
-    fn extract_vars(&mut self, s: &mut Stmt) {
-        let mut found_other = false;
-        if let Stmt::Block(bs) = s {
-            // Extract variables without
-            for stmt in &mut bs.stmts {
-                match stmt {
-                    Stmt::Decl(Decl::Var(
-                        v @ VarDecl {
-                            kind: VarDeclKind::Var,
-                            ..
-                        },
-                    )) => {
-                        for decl in &mut v.decls {
-                            if decl.init.is_some() {
-                                continue;
-                            }
-                            self.changed = true;
-                            tracing::debug!("sequences: Hoisting `var` without init");
-                            let s = Stmt::Decl(Decl::Var(VarDecl {
-                                span: v.span,
-                                kind: VarDeclKind::Var,
-                                declare: false,
-                                decls: vec![decl.take()],
-                            }));
-                            if found_other {
-                                self.append_stmts.push(s);
-                            } else {
-                                self.prepend_stmts.push(s);
-                            }
-                        }
-
-                        v.decls.retain(|v| !v.name.is_invalid());
-                    }
-                    _ => {
-                        found_other = true;
-                    }
-                }
-            }
-
-            bs.stmts.retain(|s| match s {
-                Stmt::Empty(..) => false,
-                Stmt::Decl(Decl::Var(v)) => !v.decls.is_empty(),
-                _ => true,
-            });
-        }
+        self.prepend_stmts = old_prepend;
+        self.append_stmts = old_append;
     }
 
     ///
@@ -573,7 +526,7 @@ where
                     if lhs.sym == last_id.sym && lhs.span.ctxt == last_id.span.ctxt {
                         e.exprs.pop();
                         self.changed = true;
-                        tracing::debug!("sequences: Shifting assignment");
+                        report_change!("sequences: Shifting assignment");
                     }
                 };
             }
@@ -595,7 +548,7 @@ where
         if let Some(last) = e.exprs.last() {
             if is_pure_undefined(last) {
                 self.changed = true;
-                tracing::debug!("sequences: Shifting void");
+                report_change!("sequences: Shifting void");
 
                 e.exprs.pop();
                 let last = e.exprs.last_mut().unwrap();
@@ -648,16 +601,12 @@ where
         T: ModuleItemExt,
     {
         if !self.options.sequences() && !self.options.collapse_vars {
-            if cfg!(feature = "debug") {
-                tracing::trace!("sequences: [x] Disabled");
-            }
+            log_abort!("sequences: [x] Disabled");
             return;
         }
 
         if self.ctx.in_top_level() && !self.options.top_level() {
-            if cfg!(feature = "debug") {
-                tracing::trace!("sequences: [x] Top level");
-            }
+            log_abort!("sequences: [x] Top level");
             return;
         }
 
@@ -1080,19 +1029,19 @@ where
             Expr::Update(..) | Expr::Arrow(..) | Expr::Fn(..) => return Ok(false),
 
             Expr::Cond(b) => {
-                tracing::trace!("seq: Try test of cond");
+                trace_op!("seq: Try test of cond");
                 return self.merge_sequential_expr(a, &mut *b.test);
             }
 
             Expr::Unary(b) => {
-                tracing::trace!("seq: Try arg of unary");
+                trace_op!("seq: Try arg of unary");
                 return self.merge_sequential_expr(a, &mut b.arg);
             }
 
             Expr::Bin(BinExpr {
                 op, left, right, ..
             }) => {
-                tracing::trace!("seq: Try left of bin");
+                trace_op!("seq: Try left of bin");
                 if self.merge_sequential_expr(a, &mut **left)? {
                     return Ok(true);
                 }
@@ -1106,12 +1055,12 @@ where
                     _ => {}
                 }
 
-                tracing::trace!("seq: Try right of bin");
+                trace_op!("seq: Try right of bin");
                 return self.merge_sequential_expr(a, &mut **right);
             }
 
             Expr::Member(MemberExpr { obj, prop, .. }) if !prop.is_computed() => {
-                tracing::trace!("seq: Try object of member");
+                trace_op!("seq: Try object of member");
                 return self.merge_sequential_expr(a, &mut **obj);
             }
 
@@ -1120,7 +1069,7 @@ where
                 prop: MemberProp::Computed(c),
                 ..
             }) => {
-                tracing::trace!("seq: Try object of member (computed)");
+                trace_op!("seq: Try object of member (computed)");
                 if self.merge_sequential_expr(a, &mut **obj)? {
                     return Ok(true);
                 }
@@ -1129,7 +1078,7 @@ where
                     return Ok(false);
                 }
 
-                tracing::trace!("seq: Try prop of member (computed)");
+                trace_op!("seq: Try prop of member (computed)");
                 return self.merge_sequential_expr(a, &mut c.expr);
             }
 
@@ -1137,14 +1086,14 @@ where
                 prop: SuperProp::Computed(c),
                 ..
             }) => {
-                tracing::trace!("seq: Try prop of member (computed)");
+                trace_op!("seq: Try prop of member (computed)");
                 return self.merge_sequential_expr(a, &mut c.expr);
             }
 
             Expr::Assign(b @ AssignExpr { op: op!("="), .. }) => {
                 match &mut b.left {
                     PatOrExpr::Expr(b_left) => {
-                        tracing::trace!("seq: Try lhs of assign");
+                        trace_op!("seq: Try lhs of assign");
                         if self.merge_sequential_expr(a, &mut **b_left)? {
                             return Ok(true);
                         }
@@ -1159,7 +1108,7 @@ where
                     }
                     PatOrExpr::Pat(b_left) => match &mut **b_left {
                         Pat::Expr(b_left) => {
-                            tracing::trace!("seq: Try lhs of assign");
+                            trace_op!("seq: Try lhs of assign");
                             if self.merge_sequential_expr(a, &mut **b_left)? {
                                 return Ok(true);
                             }
@@ -1180,7 +1129,7 @@ where
                     return Ok(false);
                 }
 
-                tracing::trace!("seq: Try rhs of assign");
+                trace_op!("seq: Try rhs of assign");
                 return self.merge_sequential_expr(a, &mut b.right);
             }
 
@@ -1199,7 +1148,7 @@ where
                     return Err(());
                 }
 
-                tracing::trace!("seq: Try rhs of assign with op");
+                trace_op!("seq: Try rhs of assign with op");
                 return self.merge_sequential_expr(a, &mut b.right);
             }
 
@@ -1207,7 +1156,7 @@ where
                 for elem in &mut b.elems {
                     match elem {
                         Some(elem) => {
-                            tracing::trace!("seq: Try element of array");
+                            trace_op!("seq: Try element of array");
                             if self.merge_sequential_expr(a, &mut elem.expr)? {
                                 return Ok(true);
                             }
@@ -1230,7 +1179,7 @@ where
                 ..
             }) => {
                 let is_this_undefined = b_callee.is_ident();
-                tracing::trace!("seq: Try callee of call");
+                trace_op!("seq: Try callee of call");
                 if self.merge_sequential_expr(a, &mut **b_callee)? {
                     if is_this_undefined {
                         if let Expr::Member(..) = &**b_callee {
@@ -1239,7 +1188,7 @@ where
                                 value: 0.0,
                                 raw: None,
                             })));
-                            tracing::debug!("injecting zero to preserve `this` in call");
+                            report_change!("injecting zero to preserve `this` in call");
 
                             *b_callee = Box::new(Expr::Seq(SeqExpr {
                                 span: b_callee.span(),
@@ -1256,7 +1205,7 @@ where
                 }
 
                 for arg in b_args {
-                    tracing::trace!("seq: Try arg of call");
+                    trace_op!("seq: Try arg of call");
                     if self.merge_sequential_expr(a, &mut arg.expr)? {
                         return Ok(true);
                     }
@@ -1272,7 +1221,7 @@ where
             Expr::New(NewExpr {
                 callee: b_callee, ..
             }) => {
-                tracing::trace!("seq: Try callee of new");
+                trace_op!("seq: Try callee of new");
                 if self.merge_sequential_expr(a, &mut **b_callee)? {
                     return Ok(true);
                 }
@@ -1282,7 +1231,7 @@ where
 
             Expr::Seq(SeqExpr { exprs: b_exprs, .. }) => {
                 for b_expr in b_exprs {
-                    tracing::trace!("seq: Try elem of seq");
+                    trace_op!("seq: Try elem of seq");
 
                     if self.merge_sequential_expr(a, &mut **b_expr)? {
                         return Ok(true);
@@ -1358,22 +1307,20 @@ where
             _ => {}
         }
 
-        if cfg!(feature = "debug") {
-            match a {
-                Mergable::Var(a) => {
-                    tracing::trace!(
-                        "sequences: Trying to merge `{}` => `{}`",
-                        crate::debug::dump(&**a, false),
-                        crate::debug::dump(&*b, false)
-                    );
-                }
-                Mergable::Expr(a) => {
-                    tracing::trace!(
-                        "sequences: Trying to merge `{}` => `{}`",
-                        crate::debug::dump(&**a, false),
-                        crate::debug::dump(&*b, false)
-                    );
-                }
+        match a {
+            Mergable::Var(a) => {
+                trace_op!(
+                    "sequences: Trying to merge `{}` => `{}`",
+                    crate::debug::dump(&**a, false),
+                    crate::debug::dump(&*b, false)
+                );
+            }
+            Mergable::Expr(a) => {
+                trace_op!(
+                    "sequences: Trying to merge `{}` => `{}`",
+                    crate::debug::dump(&**a, false),
+                    crate::debug::dump(&*b, false)
+                );
             }
         }
 
@@ -1422,15 +1369,13 @@ where
                     };
                     b.visit_with(&mut v);
                     if v.expr_usage != 1 || v.pat_usage != 0 {
-                        if cfg!(feature = "debug") {
-                            tracing::trace!(
-                                "[X] sequences: Aborting merging of an update expression because \
-                                 of usage counts ({}, ref = {}, pat = {})",
-                                a_id,
-                                v.expr_usage,
-                                v.pat_usage
-                            );
-                        }
+                        log_abort!(
+                            "sequences: Aborting merging of an update expression because of usage \
+                             counts ({}, ref = {}, pat = {})",
+                            a_id,
+                            v.expr_usage,
+                            v.pat_usage
+                        );
 
                         return Ok(false);
                     }
@@ -1465,7 +1410,7 @@ where
                     });
                     if replaced {
                         self.changed = true;
-                        tracing::debug!(
+                        report_change!(
                             "sequences: Merged update expression into another expression",
                         );
 
@@ -1497,7 +1442,7 @@ where
                         let left_id = match left.as_ident() {
                             Some(v) => v,
                             None => {
-                                tracing::trace!("[X] sequences: Aborting because lhs is not an id");
+                                log_abort!("sequences: Aborting because lhs is not an id");
                                 return Ok(false);
                             }
                         };
@@ -1513,8 +1458,8 @@ where
                             }
 
                             if usage.declared_as_fn_expr {
-                                tracing::trace!(
-                                    "sequences: [X] Declared as fn expr ({}, {:?})",
+                                log_abort!(
+                                    "sequences: Declared as fn expr ({}, {:?})",
                                     left_id.sym,
                                     left_id.span.ctxt
                                 );
@@ -1592,7 +1537,7 @@ where
                 }
 
                 if used_by_b.contains(id) {
-                    tracing::trace!("[X] sequences: Aborting because of deps");
+                    log_abort!("[X] sequences: Aborting because of deps");
                     return Err(());
                 }
             }
@@ -1607,22 +1552,20 @@ where
             };
             b.visit_with(&mut v);
             if v.expr_usage != 1 || v.pat_usage != 0 {
-                if cfg!(feature = "debug") {
-                    tracing::trace!(
-                        "[X] sequences: Aborting because of usage counts ({}{:?}, ref = {}, pat = \
-                         {})",
-                        left_id.sym,
-                        left_id.span.ctxt,
-                        v.expr_usage,
-                        v.pat_usage
-                    );
-                }
+                log_abort!(
+                    "sequences: Aborting because of usage counts ({}{:?}, ref = {}, pat = {})",
+                    left_id.sym,
+                    left_id.span.ctxt,
+                    v.expr_usage,
+                    v.pat_usage
+                );
+
                 return Ok(false);
             }
         }
 
         self.changed = true;
-        tracing::debug!(
+        report_change!(
             "sequences: Inlining sequential expressions (`{}{:?}`)",
             left_id.sym,
             left_id.span.ctxt
@@ -1635,9 +1578,7 @@ where
 
         replace_id_with_expr(b, left_id.to_id(), to);
 
-        if cfg!(feature = "debug") {
-            tracing::debug!("sequences: [Changed] {}", dump(&*b, false));
-        }
+        dump_change_detail!("sequences: {}", dump(&*b, false));
 
         Ok(true)
     }
