@@ -49,7 +49,7 @@ where
                     self.drop_unused_vars(var.span, &mut var.name, Some(init));
 
                     if var.name.is_invalid() {
-                        tracing::debug!("unused: Removing an unused variable declarator");
+                        report_change!("unused: Removing an unused variable declarator");
                         let side_effects = self.ignore_return_value(init).map(Box::new);
                         if let Some(e) = side_effects {
                             *storage_for_side_effects = Some(e);
@@ -118,9 +118,7 @@ where
             }
         }
 
-        if cfg!(feature = "debug") {
-            tracing::trace!("unused: drop_unused_vars({})", dump(&*name, false));
-        }
+        trace_op!("unused: drop_unused_vars({})", dump(&*name, false));
 
         // Top-level
         if !has_mark {
@@ -128,12 +126,10 @@ where
                 Some(VarDeclKind::Var) => {
                     if !self.options.top_level() && self.options.top_retain.is_empty() {
                         if self.ctx.in_top_level() {
-                            if cfg!(feature = "debug") {
-                                tracing::trace!(
-                                    "unused: [X] Preserving `var` `{}` because it's top-level",
-                                    dump(&*name, false)
-                                );
-                            }
+                            log_abort!(
+                                "unused: [X] Preserving `var` `{}` because it's top-level",
+                                dump(&*name, false)
+                            );
 
                             return;
                         }
@@ -141,13 +137,11 @@ where
                 }
                 Some(VarDeclKind::Let) | Some(VarDeclKind::Const) => {
                     if !self.options.top_level() && self.ctx.is_top_level_for_block_level_vars() {
-                        if cfg!(feature = "debug") {
-                            tracing::trace!(
-                                "unused: [X] Preserving block scoped var `{}` because it's \
-                                 top-level",
-                                dump(&*name, false)
-                            );
-                        }
+                        log_abort!(
+                            "unused: Preserving block scoped var `{}` because it's top-level",
+                            dump(&*name, false)
+                        );
+
                         return;
                     }
                 }
@@ -157,12 +151,10 @@ where
 
         if let Some(scope) = self.data.scopes.get(&self.ctx.scope) {
             if scope.has_eval_call || scope.has_with_stmt {
-                if cfg!(feature = "debug") {
-                    tracing::trace!(
-                        "unused: [X] Preserving `{}` because of usages",
-                        dump(&*name, false)
-                    );
-                }
+                log_abort!(
+                    "unused: Preserving `{}` because of usages",
+                    dump(&*name, false)
+                );
                 return;
             }
         }
@@ -192,16 +184,12 @@ where
         i: &mut Ident,
         init: Option<&mut Expr>,
     ) {
-        if cfg!(debug_assertions) {
-            tracing::trace!("unused: Checking identifier `{}`", i);
-        }
+        trace_op!("unused: Checking identifier `{}`", i);
 
         if !parent_span.has_mark(self.marks.non_top_level)
             && self.options.top_retain.contains(&i.sym)
         {
-            if cfg!(feature = "debug") {
-                tracing::trace!("unused: [X] Top-retain")
-            }
+            log_abort!("unused: [X] Top-retain");
             return;
         }
 
@@ -213,7 +201,7 @@ where
                 && !v.declared_as_catch_param
             {
                 self.changed = true;
-                tracing::debug!(
+                report_change!(
                     "unused: Dropping a variable '{}{:?}' because it is not used",
                     i.sym,
                     i.span.ctxt
@@ -244,12 +232,10 @@ where
                 }
             }
 
-            if cfg!(feature = "debug") {
-                tracing::trace!(
-                    "unused: Cannot drop ({}) because it's used",
-                    dump(&*i, false)
-                );
-            }
+            log_abort!(
+                "unused: Cannot drop ({}) because it's used",
+                dump(&*i, false)
+            );
         }
     }
 
@@ -331,9 +317,7 @@ where
             return;
         }
 
-        if cfg!(feature = "debug") {
-            tracing::trace!("unused: take_pat_if_unused({})", dump(&*name, false));
-        }
+        trace_op!("unused: take_pat_if_unused({})", dump(&*name, false));
 
         if !name.is_ident() {
             // TODO: Use smart logic
@@ -488,7 +472,7 @@ where
                     .unwrap_or(false)
                 {
                     self.changed = true;
-                    tracing::debug!(
+                    report_change!(
                         "unused: Dropping a decl '{}{:?}' because it is not used",
                         ident.sym,
                         ident.span.ctxt
@@ -525,7 +509,7 @@ where
                 // Update is counted as usage
                 if var.declared && var.is_fn_local && var.usage_count == 1 {
                     self.changed = true;
-                    tracing::debug!(
+                    report_change!(
                         "unused: Dropping an update '{}{:?}' because it is not used",
                         arg.sym,
                         arg.span.ctxt
@@ -570,7 +554,7 @@ where
                     // TODO: We don't need fn_local check
                     if var.declared && var.is_fn_local && var.usage_count == 1 {
                         self.changed = true;
-                        tracing::debug!(
+                        report_change!(
                             "unused: Dropping an op-assign '{}{:?}' because it is not used",
                             left.id.sym,
                             left.id.span.ctxt
@@ -616,23 +600,19 @@ where
             })
             .used_arguments;
 
-        if cfg!(feature = "debug") {
-            tracing::trace!(
-                "unused: drop_unused_assignments: Target: `{}`",
-                dump(&assign.left, false)
-            )
-        }
+        trace_op!(
+            "unused: drop_unused_assignments: Target: `{}`",
+            dump(&assign.left, false)
+        );
 
         if !has_mark
             && (!self.options.top_level() && self.options.top_retain.is_empty())
             && self.ctx.in_top_level()
         {
-            if cfg!(feature = "debug") {
-                tracing::trace!(
-                    "unused: Preserving assignment to `{}` because it's top-level",
-                    dump(&assign.left, false)
-                )
-            }
+            log_abort!(
+                "unused: Preserving assignment to `{}` because it's top-level",
+                dump(&assign.left, false)
+            );
             return;
         }
 
@@ -640,12 +620,10 @@ where
 
         match &mut assign.left {
             PatOrExpr::Expr(_) => {
-                if cfg!(feature = "debug") {
-                    tracing::trace!(
-                        "unused: Preserving assignment to `{}` because it's an expression",
-                        dump(&assign.left, false)
-                    )
-                }
+                log_abort!(
+                    "unused: Preserving assignment to `{}` because it's an expression",
+                    dump(&assign.left, false)
+                );
             }
             PatOrExpr::Pat(left) => {
                 if let Pat::Ident(i) = &**left {
@@ -659,7 +637,7 @@ where
                             && var.declared
                             && (!var.declared_as_fn_param || !used_arguments || self.ctx.in_strict)
                         {
-                            tracing::debug!(
+                            report_change!(
                                 "unused: Dropping assignment to var '{}{:?}', which is never used",
                                 i.id.sym,
                                 i.id.span.ctxt
@@ -674,13 +652,11 @@ where
                                 *e = *assign.right.take();
                             }
                         } else {
-                            if cfg!(feature = "debug") {
-                                tracing::trace!(
-                                    "unused: Preserving assignment to `{}` because of usage: {:?}",
-                                    dump(&assign.left, false),
-                                    var
-                                )
-                            }
+                            log_abort!(
+                                "unused: Preserving assignment to `{}` because of usage: {:?}",
+                                dump(&assign.left, false),
+                                var
+                            )
                         }
                     }
                 }
@@ -709,7 +685,7 @@ where
 
             if can_remove_ident {
                 self.changed = true;
-                tracing::debug!("Removing ident of an class / function expression");
+                report_change!("Removing ident of an class / function expression");
                 *name = None;
             }
         }
@@ -731,7 +707,7 @@ where
                     if let Some(usage) = self.data.vars.get(&name.to_id()) {
                         if usage.is_fn_local && usage.declared_as_fn_param {
                             d.name.take();
-                            tracing::debug!(
+                            report_change!(
                                 "Removing a variable statement because it's a function parameter"
                             );
                             self.changed = true;
@@ -762,7 +738,7 @@ where
             }
 
             self.changed = true;
-            tracing::debug!(
+            report_change!(
                 "unused: Removing the name of a function expression because it's not used by it'"
             );
             f.ident = None;
