@@ -27,7 +27,7 @@ impl Pure<'_> {
         if let Some(e) = s.arg.as_deref() {
             if is_pure_undefined(e) {
                 self.changed = true;
-                tracing::debug!("Dropped `undefined` from `return undefined`");
+                report_change!("Dropped `undefined` from `return undefined`");
                 s.arg.take();
             }
         }
@@ -36,7 +36,7 @@ impl Pure<'_> {
     pub(super) fn remove_useless_return(&mut self, stmts: &mut Vec<Stmt>) {
         if let Some(Stmt::Return(ReturnStmt { arg: None, .. })) = stmts.last() {
             self.changed = true;
-            tracing::debug!("misc: Removing useless return");
+            report_change!("misc: Removing useless return");
             stmts.pop();
         }
     }
@@ -82,9 +82,7 @@ impl Pure<'_> {
             Stmt::Block(s) => self.drop_return_value(&mut s.stmts),
             Stmt::Return(ret) => {
                 self.changed = true;
-                if cfg!(feature = "debug") {
-                    tracing::trace!("Dropping `return` token");
-                }
+                report_change!("Dropping `return` token");
 
                 let span = ret.span;
                 match ret.arg.take() {
@@ -177,7 +175,7 @@ impl Pure<'_> {
             }
 
             Expr::Call(CallExpr { span, args, .. }) if span.has_mark(self.marks.pure) => {
-                tracing::debug!("ignore_return_value: Dropping a pure call");
+                report_change!("ignore_return_value: Dropping a pure call");
                 self.changed = true;
 
                 let new = self.make_ignored_expr(args.take().into_iter().map(|arg| arg.expr));
@@ -191,7 +189,7 @@ impl Pure<'_> {
                 tpl: Tpl { exprs, .. },
                 ..
             }) if span.has_mark(self.marks.pure) => {
-                tracing::debug!("ignore_return_value: Dropping a pure call");
+                report_change!("ignore_return_value: Dropping a pure call");
                 self.changed = true;
 
                 let new = self.make_ignored_expr(exprs.take().into_iter());
@@ -201,7 +199,7 @@ impl Pure<'_> {
             }
 
             Expr::New(NewExpr { span, args, .. }) if span.has_mark(self.marks.pure) => {
-                tracing::debug!("ignore_return_value: Dropping a pure call");
+                report_change!("ignore_return_value: Dropping a pure call");
                 self.changed = true;
 
                 let new =
@@ -232,13 +230,13 @@ impl Pure<'_> {
                     || (self.options.unused && opts.drop_global_refs_if_unused)
                 {
                     if is_global_var(&i.sym) {
-                        tracing::debug!("Dropping a reference to a global variable");
+                        report_change!("Dropping a reference to a global variable");
                         *e = Expr::Invalid(Invalid { span: DUMMY_SP });
                         return;
                     }
                 }
             } else {
-                tracing::debug!("Dropping an identifier as it's declared");
+                report_change!("Dropping an identifier as it's declared");
                 *e = Expr::Invalid(Invalid { span: DUMMY_SP });
                 return;
             }
@@ -262,7 +260,7 @@ impl Pure<'_> {
                     );
 
                     if arg.is_invalid() {
-                        tracing::debug!("Dropping an unary expression");
+                        report_change!("Dropping an unary expression");
                         *e = Expr::Invalid(Invalid { span: DUMMY_SP });
                         return;
                     }
@@ -277,7 +275,7 @@ impl Pure<'_> {
                     self.ignore_return_value(&mut be.right, opts);
 
                     if be.right.is_invalid() {
-                        tracing::debug!("Dropping the RHS of a binary expression ('&&' / '||')");
+                        report_change!("Dropping the RHS of a binary expression ('&&' / '||')");
                         *e = *be.left.take();
                         return;
                     }
@@ -300,7 +298,7 @@ impl Pure<'_> {
                 Expr::Ident(i) => {
                     if let Some(bindings) = self.bindings.as_deref() {
                         if bindings.contains(&i.to_id()) {
-                            tracing::debug!("Dropping an identifier as it's declared");
+                            report_change!("Dropping an identifier as it's declared");
 
                             self.changed = true;
                             *e = Expr::Invalid(Invalid { span: DUMMY_SP });
@@ -310,7 +308,7 @@ impl Pure<'_> {
                 }
 
                 Expr::Lit(Lit::Null(..) | Lit::BigInt(..) | Lit::Bool(..) | Lit::Regex(..)) => {
-                    tracing::debug!("Dropping literals");
+                    report_change!("Dropping literals");
 
                     self.changed = true;
                     *e = Expr::Invalid(Invalid { span: DUMMY_SP });
@@ -376,7 +374,7 @@ impl Pure<'_> {
 
                     if matches!(*bin.left, Expr::Await(..) | Expr::Update(..)) {
                         self.changed = true;
-                        tracing::debug!("ignore_return_value: Compressing binary as seq");
+                        report_change!("ignore_return_value: Compressing binary as seq");
                         *e = Expr::Seq(SeqExpr {
                             span,
                             exprs: vec![bin.left.take(), bin.right.take()],
